#include "avo.hh"
#include "nto.hh"
#include "rto.hh"
#include "util.hh"
#include <typeinfo>

AttributeValueOptimizer::AttributeValueOptimizer()
{
  name = "AttributeValueOptimizer";
}


RedundantTagOptimizer rto;
NeighbouringTagOptimizer nto;

const int ntypes = 4;
const SMLTagType types[] = { ST_I, ST_B, ST_S, ST_TT };

const int ncolours = 8;
const char colours[] = "rgbcmykw";
const SMLTagType colourtags[] = { ST_r, ST_g, ST_b, ST_c, ST_m, ST_y, ST_k, ST_w };

const SMLTagType sizetags[] = { ST_0, ST_1, ST_2, ST_3, ST_4, ST_5, ST_6, ST_7, ST_8, ST_9 };

SMLTagType GetColourTagType( char c )
{
  switch(c) {
  case 'r': return ST_r;
  case 'g': return ST_g;
  case 'b': return ST_b;
  case 'c': return ST_c;
  case 'm': return ST_m;
  case 'y': return ST_y;
  case 'k': return ST_k;
  case 'w': return ST_w;
  }

  return ST_NONE;
}

void
AttributeValueOptimizer::Optimize( SMLTag& doc )
{
  for( int i = 0; i < ntypes; ++i ) {
    Suboptimize( doc, types[i] );
    if( ContainsText(&doc) && IsAllInAttr( &doc, types[i] ) ) {
      SMLTag* tag = new SMLTag( types[i] );
      tag->subs.assign( doc.subs.begin(), doc.subs.end() );
      doc.subs.clear();
      doc.subs.push_back(tag);
    }

    doc.CalculateContexts();
    rto.Optimize(doc);
    nto.Optimize(doc);
  }

  for( int i = 0; i < ncolours; ++i ) {
    Suboptimize2( doc, colours[i] );
    if( ContainsText(&doc) && IsAllInColour( &doc, colours[i] ) ) {
      SMLTag* tag = new SMLTag( colourtags[i] );
      tag->subs.assign( doc.subs.begin(), doc.subs.end() );
      doc.subs.clear();
      doc.subs.push_back(tag);
    }

    doc.CalculateContexts();
    rto.Optimize(doc);
    nto.Optimize(doc);
  }

  for( int i = 0; i < 10; ++i ) {
    Suboptimize3( doc, i );
    if( ContainsText(&doc) && IsAllInSize( &doc, i ) ) {
      SMLTag* tag = new SMLTag( sizetags[i] );
      tag->subs.assign( doc.subs.begin(), doc.subs.end() );
      doc.subs.clear();
      doc.subs.push_back(tag);
    }

    doc.CalculateContexts();
    rto.Optimize(doc);
    nto.Optimize(doc);
  }
}

void
AttributeValueOptimizer::Suboptimize( SMLTag& doc, SMLTagType type )
{
  for( SMLEntity_list_i i = doc.subs.begin(); i != doc.subs.end(); ++i ) {
    if( typeid(**i) == typeid(SMLTag) ) {
      SMLTag* tag = dynamic_cast<SMLTag*>(*i);
      Suboptimize( *tag, type );
      if( tag->tag != type && ContainsText(tag) && IsAllInAttr( tag, type ) ) {
	SMLTag* tag2 = new SMLTag( type );
	tag2->subs.push_back(tag);
	*i = tag2;
      }
    }
  }
}

void
AttributeValueOptimizer::Suboptimize2( SMLTag& doc, char c )
{
  for( SMLEntity_list_i i = doc.subs.begin(); i != doc.subs.end(); ++i ) {
    if( typeid(**i) == typeid(SMLTag) ) {
      SMLTag* tag = dynamic_cast<SMLTag*>(*i);
      Suboptimize2( *tag, c );
      if( tag->cxt.color != c && ContainsText(tag) && IsAllInColour( tag, c ) ) {
	SMLTag* tag2 = new SMLTag( GetColourTagType(c) );
	tag2->subs.push_back(tag);
	*i = tag2;
      }
    }
  }
}

void
AttributeValueOptimizer::Suboptimize3( SMLTag& doc, int size )
{
  for( SMLEntity_list_i i = doc.subs.begin(); i != doc.subs.end(); ++i ) {
    if( typeid(**i) == typeid(SMLTag) ) {
      SMLTag* tag = dynamic_cast<SMLTag*>(*i);
      Suboptimize3( *tag, size );
      if( tag->cxt.size != size && ContainsText(tag) && IsAllInSize( tag, size ) ) {
	SMLTag* tag2 = new SMLTag( sizetags[size] );
	tag2->subs.push_back(tag);
	*i = tag2;
      }
    }
  }
}
