#include "parser.hh"
#include "wso.hh"
#include "rto.hh"
#include "eto.hh"
#include "cno.hh"
#include "sno.hh"
#include "nto.hh"
#include "wss.hh"
#include "avo.hh"
#include "emo.hh"
#include "uco.hh"
#include "uso.hh"
#include "eso.hh"
#include "plo.hh"
#include "spo.hh"

int
main( int argc, char** argv )
{
  SMLTag doc( ST_NONE );
  
  //
  // Parse input.
  //
 
  Parser parser;
  parser.BuildTree(doc);


  //
  // Optimize.
  //

  Optimizer_list opts;

  opts.push_back( new WhiteSpaceShifter );
  opts.push_back( new WhiteSpaceOptimizer );
  opts.push_back( new EmptyTagOptimizer );
  opts.push_back( new RedundantTagOptimizer );
  opts.push_back( new EMSOptimizer );
  opts.push_back( new NeighbouringTagOptimizer );
  opts.push_back( new UselessColourOptimizer );
  opts.push_back( new ColourNestingOptimizer );
  opts.push_back( new UselessSizeOptimizer );
  opts.push_back( new SizeNestingOptimizer );
  opts.push_back( new EMEliminationOptimizer );
  opts.push_back( new PLOptimizer );
  opts.push_back( new SymmetricPLOptimizer );
  opts.push_back( new AttributeValueOptimizer );

  bool bSizeChanged;
  int nSize;
  do {
   bSizeChanged = false;
   nSize = doc.GetSize();
 
   for( Optimizer_list_i i = opts.begin(); i != opts.end(); ++i ) {
     (*i)->Optimize(doc);
 
     int nNewSize = doc.GetSize();
     if( nNewSize != nSize ) {
       bSizeChanged = true;
       nNewSize = nSize;
     }
   }

  } while( bSizeChanged );

  for( Optimizer_list_i i = opts.begin(); i != opts.end(); ++i )
    delete *i;


  //
  // Output.
  //

  doc.Print();

  return 0;
}
