#include "plo.hh"
#include "util.hh"
#include <typeinfo>
#include <algorithm>
using std::min;

PLOptimizer::PLOptimizer()
{
  name = "PLOptimizer";
}

void
PLOptimizer::Optimize( SMLTag& doc )
{
  for( SMLEntity_list_i i = doc.subs.begin(); i != doc.subs.end(); ++i )
    if( typeid(**i) == typeid(SMLTag) )
      Optimize( *dynamic_cast<SMLTag*>(*i) );

  if( IsPlainContext( doc.cxt ) ) {
    for( SMLEntity_list_i i = doc.subs.begin(); i != doc.subs.end(); ++i ) {
      // Is this a non-plain tag?
      if( typeid(**i) == typeid(SMLTag) && !IsPlainContext( dynamic_cast<SMLTag*>(*i)->cxt ) ) {
        SMLTag* lefttag = dynamic_cast<SMLTag*>(*i);
	// Is the next sub plain text?
	SMLEntity_list_i j = i;
	++j;
	if( j != doc.subs.end() && typeid(**j) == typeid(SMLText) ) {
	  SMLText* plaintext = dynamic_cast<SMLText*>(*j);
	  // And is the next sub a non-plain tag?
	  SMLEntity_list_i k = j;
	  ++k;
	  if( k != doc.subs.end() && typeid(**k) == typeid(SMLTag) && !IsPlainContext( dynamic_cast<SMLTag*>(*k)->cxt ) ) {
	    SMLTag* righttag = dynamic_cast<SMLTag*>(*k);
	    // Looks like we might be able to do something.
	    int depth = min( GetTagDepth(lefttag), GetTagDepth(righttag) );
	    if( depth > 1 && GetContextAtDepth( lefttag, depth ) == GetContextAtDepth( righttag, depth ) ) {
	      SMLTag *leftbottom = GetTagAtDepth( lefttag, depth ),
		     *rightbottom = GetTagAtDepth( righttag, depth ),
		     *plaintag = new SMLTag( ST_PL );
	      plaintag->subs.push_back(plaintext);
	      doc.subs.erase(j);
	      leftbottom->subs.push_back(plaintag);
	      leftbottom->subs.insert( leftbottom->subs.end(), rightbottom->subs.begin(), rightbottom->subs.end() );
	      rightbottom->subs.clear();
	      doc.subs.erase(k);
	      delete righttag;
	      lefttag->CalculateContexts();
	    }
	  }
	}
      }
    }
  }
}
