#include "sml.hh"

#include <typeinfo>
#include <iostream>
using std::cout;

SMLTagType
GetTagType( const string& tag )
{
  for( SMLTagType i = ST_NONE; i < ST_MAX; i = static_cast<SMLTagType>(i+1) )
    if( tag == SML_TAG_NAMES[i] )
      return i;

  return ST_NONE;
}

void
SMLText::Print()
{
  cout << text;
}

SMLTag::~SMLTag()
{
  DeleteSubs();
}

void
SMLTag::DeleteSubs()
{
  for( SMLEntity_list_i i = subs.begin(); i != subs.end(); ++i )
    delete *i;
  subs.clear();
}

int
SMLTag::GetSize()
{
  int size = 2 * SML_TAG_NAMES[tag].length();
  if( tag != ST_NONE )
    size += 5; // <>, </>

  for( SMLEntity_list_i i = subs.begin(); i != subs.end(); ++i )
    size += (*i)->GetSize();

  return size;
}

void
SMLTag::Print()
{
  if( tag != ST_NONE )
    cout << "<" << SML_TAG_NAMES[tag] << ">";

  for( SMLEntity_list_i i = subs.begin(); i != subs.end(); ++i )
    (*i)->Print();

  if( tag != ST_NONE )
    cout << "</" << SML_TAG_NAMES[tag] << ">";
}

SMLTag&
SMLTag::operator=( SMLTag& t )
{
  tag = t.tag;
  cxt = t.cxt;
  DeleteSubs();

  for( SMLEntity_list_i i = t.subs.begin(); i != t.subs.end(); ++i )
    if( typeid(**i) == typeid(SMLTag) ) {
      SMLTag* tag = dynamic_cast<SMLTag*>(*i);
      SMLTag* newtag = new SMLTag( tag->tag );
      *newtag = *tag;
      subs.push_back(newtag);
    }
    else {
      SMLText* text = dynamic_cast<SMLText*>(*i);
      subs.push_back( new SMLText( text->text ) );
    }

  return *this;
}

void SetContext( const SMLTagType eTagType,Context& cxt)
{
  switch( eTagType )
  {
  case ST_NONE:
    break;
  case ST_B:
    cxt.b = true;
    break;
  case ST_EM:
    if( !cxt.s ) cxt.em = !cxt.em;
    break;
  case ST_I:
    cxt.i = true;
    break;
  case ST_PL:
    cxt.u = 0;
    cxt.b = false;
    cxt.em = false;
    cxt.i = false;
    cxt.s = false; 
    cxt.tt = false;
    break;
  case ST_S:
    cxt.s = true;
    cxt.em = false;
    break;
  case ST_TT:
    cxt.tt = true;
    break;
  case ST_U:
    if( cxt.u < 3 ) cxt.u++;
    break;
  case ST_0:
    cxt.size = 0;
    break;
  case ST_1:
    cxt.size = 1;
    break;
  case ST_2:
    cxt.size = 2;
    break;
  case ST_3:
    cxt.size = 3;
    break;
  case ST_4:
    cxt.size = 4;
    break;
  case ST_5: 
    cxt.size = 5;
    break;
  case ST_6:
    cxt.size = 6;
    break;
  case ST_7:
    cxt.size = 7;
    break;
  case ST_8: 
    cxt.size = 8;
    break;
  case ST_9:
    cxt.size = 9;
    break;
  case ST_r: 
    cxt.color = 'r';
    break;
  case ST_g: 
    cxt.color = 'g';
    break;
  case ST_b: 
    cxt.color = 'b';
    break;
  case ST_c: 
    cxt.color = 'c';
    break;
  case ST_m:
    cxt.color = 'm';
    break;
  case ST_y: 
    cxt.color = 'y';
    break;
  case ST_k: 
    cxt.color = 'k';
    break;
  case ST_w:
    cxt.color = 'w';
    break;
  case ST_MAX:
    break;
  }
}

void
SMLTag::CalculateContexts()
{
  for( SMLEntity_list_i i = subs.begin(); i != subs.end(); ++i ) {
    if( typeid(**i) == typeid(SMLTag) ) {
      SMLTag* tag = dynamic_cast<SMLTag*>(*i);
      tag->cxt = cxt;
      SetContext( tag->tag, tag->cxt );
      tag->CalculateContexts();
    }
  }
}
