#ifndef SML_HH
#define SML_HH

#include "context.hh"

#include <string>
#include <list>
using std::string;
using std::list;

enum SMLTagType {
  ST_NONE,
  ST_B, ST_EM, ST_I, ST_PL, ST_S, ST_TT,
  ST_U,
  ST_0, ST_1, ST_2, ST_3, ST_4, ST_5, ST_6, ST_7, ST_8, ST_9,
  ST_r, ST_g, ST_b, ST_c, ST_m, ST_y, ST_k, ST_w,
  ST_MAX
};

const string SML_TAG_NAMES[] = {
  "",
  "B", "EM", "I", "PL", "S", "TT",
  "U",
  "0", "1", "2", "3", "4", "5", "6", "7", "8", "9",
  "r", "g", "b", "c", "m", "y", "k", "w"
};

SMLTagType GetTagType( const string& tag );

enum SMLColor { SC_NONE, SC_R, SC_G, SC_B, SC_C, SC_M, SC_Y, SC_K, SC_W, SC_MAX };

struct SMLEntity {
  virtual ~SMLEntity() { }
  virtual int GetSize() = 0;
  virtual void Print() = 0;
};

typedef list<SMLEntity*> SMLEntity_list;
typedef SMLEntity_list::iterator SMLEntity_list_i;
typedef SMLEntity_list::reverse_iterator SMLEntity_list_ri;

struct SMLText : public SMLEntity {
  SMLText( const string& text_ ) : text(text_) { }

  string text;

  int GetSize() { return text.length(); }
  void Print();
};

struct SMLTag : public SMLEntity {
  SMLTag( SMLTagType tag_ ) : tag(tag_) { }
  ~SMLTag();

  SMLTagType tag;
  SMLEntity_list subs;
  Context cxt;

  int GetSize();
  void Print();

  void DeleteSubs();
  SMLTag& operator=( SMLTag& t );

  void CalculateContexts();
};

#endif // SML_HH not defined.
