#include "spo.hh"
#include "util.hh"
#include <typeinfo>
#include <algorithm>
using std::min;

SymmetricPLOptimizer::SymmetricPLOptimizer()
{
  name = "SymmetricPLOptimizer";
}

bool
MergeSides( SMLTag* lefttag, SMLText* plaintext, SMLTag* righttag, int depth )
{
  if( lefttag->subs.empty() || righttag->subs.empty() ) return false;
  
  SMLTag *leftsub = 0, *rightsub = 0;
  if( typeid(*lefttag->subs.back()) == typeid(SMLTag) ) leftsub = dynamic_cast<SMLTag*>(lefttag->subs.back());
  if( typeid(*righttag->subs.front()) == typeid(SMLTag) ) rightsub = dynamic_cast<SMLTag*>(righttag->subs.front());

  if( leftsub && rightsub ) {
    if( leftsub->tag == rightsub->tag && IsHiddenByPL(leftsub->tag) &&
	MergeSides( leftsub, plaintext, rightsub, depth + 1 ) ) {
      righttag->subs.pop_front();
      lefttag->subs.insert( lefttag->subs.end(), righttag->subs.begin(), righttag->subs.end() );
      righttag->subs.clear();
      delete righttag;
      return true;
    }
    else if( depth >= 2 ) {
      SMLTag* plaintag = new SMLTag( ST_PL );
      plaintag->subs.push_back(plaintext);
      lefttag->subs.push_back(plaintag);
      lefttag->subs.insert( lefttag->subs.end(), righttag->subs.begin(), righttag->subs.end() );
      righttag->subs.clear();
      delete righttag;
      return true;
    }
    else
      return false;
  }
  else {
    if( depth >= 2 ) {
      SMLTag* plaintag = new SMLTag( ST_PL );
      plaintag->subs.push_back(plaintext);
      lefttag->subs.push_back(plaintag);
      lefttag->subs.insert( lefttag->subs.end(), righttag->subs.begin(), righttag->subs.end() );
      righttag->subs.clear();
      delete righttag;
      return true;
    }
    else
      return false;
  }
}

void
SymmetricPLOptimizer::Optimize( SMLTag& doc )
{
  for( SMLEntity_list_i i = doc.subs.begin(); i != doc.subs.end(); ++i )
    if( typeid(**i) == typeid(SMLTag) )
      Optimize( *dynamic_cast<SMLTag*>(*i) );

  if( IsPlainContext( doc.cxt ) ) {
    for( SMLEntity_list_i i = doc.subs.begin(); i != doc.subs.end(); ++i ) {
      // Is this a non-plain tag?
      if( typeid(**i) == typeid(SMLTag) && !IsPlainContext( dynamic_cast<SMLTag*>(*i)->cxt ) ) {
        SMLTag* lefttag = dynamic_cast<SMLTag*>(*i);
	// Is the next sub plain text?
	SMLEntity_list_i j = i;
	++j;
	if( j != doc.subs.end() && typeid(**j) == typeid(SMLText) ) {
	  SMLText* plaintext = dynamic_cast<SMLText*>(*j);
	  // And is the next sub a matching non-plain tag?
	  SMLEntity_list_i k = j;
	  ++k;
	  if( k != doc.subs.end() && typeid(**k) == typeid(SMLTag) && dynamic_cast<SMLTag*>(*k)->tag == lefttag->tag ) {
	    SMLTag* righttag = dynamic_cast<SMLTag*>(*k);
	    // Looks like we might be able to do something.
	    if( MergeSides( lefttag, plaintext, righttag, 1 ) ) {
	      doc.subs.erase(j);
	      doc.subs.erase(k);
	    }
	  }
	}
      }
    }
  }
}
