#include "util.hh"
#include <typeinfo>

bool
IsAllWhiteSpace( string& s )
{
  for( unsigned int i = 0; i < s.length(); ++i )
    if( s[i] > 0x20 ) return false;

  return true;
}

bool
ContainsText( SMLTag* tag )
{
  for( SMLEntity_list_i i = tag->subs.begin(); i != tag->subs.end(); ++i ) {
    if( typeid(**i) == typeid(SMLTag) ) {
      SMLTag* tag2 = dynamic_cast<SMLTag*>(*i);
      if( ContainsText(tag2) ) return true;
    }
    else if( !IsAllWhiteSpace( dynamic_cast<SMLText*>(*i)->text ) )
      return true;
  }

  return false;
}

bool
IsAllInAttr( SMLTag* tag, SMLTagType type )
{
  for( SMLEntity_list_i i = tag->subs.begin(); i != tag->subs.end(); ++i ) {
    if( typeid(**i) == typeid(SMLTag) ) {
      SMLTag* tag2 = dynamic_cast<SMLTag*>(*i);
      if( tag2->tag != type && !IsAllInAttr( tag2, type ) ) return false;
    }
    else if( !IsAllWhiteSpace( dynamic_cast<SMLText*>(*i)->text ) )
      return false;
  }

  return true;
}

bool
IsAllInColour( SMLTag* tag, char c )
{
  for( SMLEntity_list_i i = tag->subs.begin(); i != tag->subs.end(); ++i ) {
    if( typeid(**i) == typeid(SMLTag) ) {
      SMLTag* tag2 = dynamic_cast<SMLTag*>(*i);
      if( tag2->cxt.color != c && !IsAllInColour( tag2, c ) ) return false;
    }
    else if( !IsAllWhiteSpace( dynamic_cast<SMLText*>(*i)->text ) || tag->cxt.u > 0 )
      return false;
  }

  return true;
}

bool
IsAllInSize( SMLTag* tag, int size )
{
  for( SMLEntity_list_i i = tag->subs.begin(); i != tag->subs.end(); ++i ) {
    if( typeid(**i) == typeid(SMLTag) ) {
      SMLTag* tag2 = dynamic_cast<SMLTag*>(*i);
      if( tag2->cxt.size != size && !IsAllInSize( tag2, size ) ) return false;
    }
    else
      return false;
  }

  return true;
}

bool
IsColourUsed( SMLTag* tag, char c )
{
  for( SMLEntity_list_i i = tag->subs.begin(); i != tag->subs.end(); ++i ) {
    if( typeid(**i) == typeid(SMLTag) ) {
      SMLTag* tag2 = dynamic_cast<SMLTag*>(*i);
      if( tag2->cxt.color == c && IsColourUsed( tag2, c ) ) return true;
    }
    else if( !IsAllWhiteSpace( dynamic_cast<SMLText*>(*i)->text ) || tag->cxt.u > 0 )
      return true;
  }

  return false;
}

bool
IsSizeUsed( SMLTag* tag, int size )
{
  for( SMLEntity_list_i i = tag->subs.begin(); i != tag->subs.end(); ++i ) {
    if( typeid(**i) == typeid(SMLTag) ) {
      SMLTag* tag2 = dynamic_cast<SMLTag*>(*i);
      if( tag2->cxt.size == size && IsSizeUsed( tag2, size ) ) return true;
    }
    else
      return true;
  }

  return false;
}

bool
IsPlainContext( Context& cxt )
{
  return !cxt.s && !cxt.em && !cxt.i && !cxt.b && !cxt.tt && cxt.u == 0;
}

Context
GetContextAtDepth( SMLTag* tag, int depth )
{
  if( depth == 1 ) return tag->cxt;

  return GetContextAtDepth( dynamic_cast<SMLTag*>(tag->subs.front()), depth - 1 );
}

SMLTag*
GetTagAtDepth( SMLTag* tag, int depth )
{
  if( depth == 1 ) return tag;

  return GetTagAtDepth( dynamic_cast<SMLTag*>(tag->subs.front()), depth - 1 );
}

bool
IsHiddenByPL( SMLTagType t )
{
  return t == ST_S || t == ST_EM || t == ST_I || t == ST_B || t == ST_TT || t == ST_U;
}

int
GetTagDepth( SMLTag* tag )
{
  if( tag->subs.size() != 1 ) return 1;
  
  if( typeid(*tag->subs.front()) == typeid(SMLText) ) return 1;

  SMLTag* tag2 = dynamic_cast<SMLTag*>(tag->subs.front());
  if( !IsHiddenByPL(tag2->tag) ) return 1;

  return 1 + GetTagDepth( dynamic_cast<SMLTag*>(tag->subs.front()) );
}
